package org.processmining.plugins.bpsim;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.models.graphbased.directed.bpmn.elements.Flow;
import org.processmining.plugins.bpsim.constantparameter.FloatingParameter;
import org.processmining.plugins.bpsim.elementparameter.ControlParameters;
import org.processmining.plugins.bpsim.parameter.Parameter;
import org.processmining.plugins.bpsim.parameter.ParameterValue;
import org.processmining.plugins.bpsim.scenario.ElementParameters;
import org.processmining.plugins.bpsim.scenario.Scenario;
import org.processmining.plugins.properties.processmodel.impl.DataValueMatrix;
import org.processmining.plugins.properties.processmodel.impl.InitialDataValue;
import org.processmining.plugins.properties.processmodel.impl.OutgoingProbability;
import org.processmining.processtree.Block;
import org.processmining.processtree.Block.Seq;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Expression;
import org.processmining.processtree.Node;
import org.processmining.processtree.Variable;
import org.processmining.processtree.impl.ExpressionImpl;

public class Auxiliary {
	
	
	public static void setProbabilityElement(BPMNDiagram bpmnDiagram, Node node, Scenario scenario, ProcessTree2BPSimConverter converter) throws IllegalAccessException, InstantiationException{
		// we do not have the individual edges, so we have to deduce the edge in between these nodes, let's first get the parent of this node
		Node localNode = node;
		Edge localEdge;
		if(node.getIncomingEdges().size() > 0){
			localEdge = node.getIncomingEdges().get(0);
		}
		else{
			// we have the root
			return;
		}
		while(localEdge.getSource() instanceof Seq && localEdge.getSource().getOutgoingEdges().indexOf(localEdge) == 0){
			localNode = localEdge.getSource();
			if(localNode.getIncomingEdges().size() > 0){
				localEdge = localNode.getIncomingEdges().get(0);
			}
			else{
				// we have the root
				// so let's stop
				return;
			}
		}
		
		localNode = localEdge.getSource();
		// so we have a node for which we need a probability
		// we have to set an expression for this node
		Double probability = determineProbability(localEdge);
		if(!probability.equals(-1.0)){
			// we have to normalise this probability
			Double totalProb = 0d;
			for(Edge e: ((Block)localNode).getOutgoingEdges()){
				Double probEdge = determineProbability(e);
				if(probEdge != -1){
					totalProb += probEdge;
				}
			}
			if(totalProb > 1d){
				// we are done everything is fine
				totalProb = 1d;
			}
			// first find the element parameters
			String nodeId = (String) converter.getBPMNElement(node.getID())[0];
			String parentId = (String) converter.getBPMNElement(localNode.getID())[0];
			
			String flowId = "";
			for(Flow flow: bpmnDiagram.getFlows()){
				if(flow.getSource().getId().toString().equalsIgnoreCase(parentId) && flow.getTarget().getId().toString().equalsIgnoreCase(nodeId)){
					// we have the flow
					flowId = flow.getEdgeID().toString();
				}
			}
			
			ElementParameters elemParamEdge = null;
			for(ElementParameters elemParam: scenario.getElementParameter()){
				if(elemParam.getElementRefnodeID().equalsIgnoreCase(flowId)){
					elemParamEdge = elemParam;
				}
			}
			if(elemParamEdge == null){
				// make a new one
				elemParamEdge = new ElementParameters();
				elemParamEdge.setElementRefnodeID(flowId);
				scenario.getElementParameter().add(elemParamEdge);
			}
			
			
			ControlParameters cp = elemParamEdge.getControlParameters();
			if(cp == null){
				cp = new ControlParameters();
				elemParamEdge.setControlParameters(cp);
			}
			Parameter prob = new Parameter();
			Collection<ParameterValue> values = new ArrayList<ParameterValue>();
			FloatingParameter value = new FloatingParameter();
			value.setValue(probability.floatValue() / totalProb.floatValue());
			values.add(value);
			prob.setValue(values);
			cp.setProbability(prob);
		}
	}
	
	public static Double determineProbability(Edge e) throws IllegalAccessException, InstantiationException{
		if(e.getIndependentProperty(OutgoingProbability.class) != null){
			return (Double) e.getIndependentProperty(OutgoingProbability.class);
		}
		if(e.hasExpression()){
			ExpressionGrammar expressionGrammar = parseExpression(e.getExpression().getVariables(), e.getExpression());
			return determineProbability(expressionGrammar);
		}
		return -1.0;
	}
	
	private static ExpressionGrammar parseExpression(Collection<Variable> variables, Expression expr){
		// find the top most operator
		// just split the expression and see how many brackets are on each side

		ExpressionGrammar expressionGrammar = null;
		if(isCorrectOperator(expr.getExpression().split("=="))){
			expressionGrammar = new ExpressionLiteral();
			String leftElement = expr.getExpression().split("==")[0];
			// remove any ""quotes
			leftElement = leftElement.replace("\"", "");
			// remove any brackets
			leftElement = leftElement.replace("(", "");
			String rightElement = expr.getExpression().split("==")[1];
			rightElement = rightElement.replace("\"", "");
			// remove any brackets
			rightElement = rightElement.replace(")", "");
			// now check which one is a variable and which one is a value
			for(Variable var: variables){
				if(var.getName().equalsIgnoreCase(leftElement)){
					expressionGrammar.addVariable(var);
					expressionGrammar.addValues(rightElement);
					break;
				}
				else if(var.getName().equalsIgnoreCase(rightElement)){
					expressionGrammar.addVariable(var);
					expressionGrammar.addValues(leftElement);
				}
			}
			expressionGrammar.setOperator(ExpressionOperator.equals);
		}
		else if(isCorrectOperator(expr.getExpression().split("!="))){
			expressionGrammar = new ExpressionLiteral();
			String leftElement = expr.getExpression().split("!=")[0];
			// remove any ""quotes
			leftElement = leftElement.replace("\"", "");
			// remove any brackets
			leftElement = leftElement.replace("(", "");
			String rightElement = expr.getExpression().split("!=")[1];
			rightElement = rightElement.replace("\"", "");
			// remove any brackets
			rightElement = rightElement.replace(")", "");
			// now check which one is a variable and which one is a value
			for(Variable var: variables){
				if(var.getName().equalsIgnoreCase(leftElement)){
					expressionGrammar.addVariable(var);
					expressionGrammar.addValues(rightElement);
					break;
				}
				else if(var.getName().equalsIgnoreCase(rightElement)){
					expressionGrammar.addVariable(var);
					expressionGrammar.addValues(leftElement);
				}
			}
			expressionGrammar.setOperator(ExpressionOperator.notEquals);
		}
		else if(isCorrectOperator(expr.getExpression().split("\\&\\&"))){
			expressionGrammar = new ExpressionComposed();
			String leftElement = expr.getExpression().split("\\&\\&")[0];
			// remove any ""quotes
			leftElement.replace("\"", "");
			String rightElement = expr.getExpression().split("\\&\\&")[1];
			rightElement.replace("\"", "");
			expressionGrammar.setLeftClause(parseExpression(variables, new ExpressionImpl(leftElement, (Variable[]) expr.getVariables().toArray())));
			expressionGrammar.setRightClause(parseExpression(variables, new ExpressionImpl(rightElement, (Variable[]) expr.getVariables().toArray())));
			expressionGrammar.setOperator(ExpressionOperator.and);
		}
		else if(isCorrectOperator(expr.getExpression().split("\\|\\|"))){
			expressionGrammar = new ExpressionComposed();
			String leftElement = expr.getExpression().split("\\|\\|")[0];
			// remove any ""quotes
			leftElement.replace("\"", "");
			String rightElement = expr.getExpression().split("\\|\\|")[1];
			rightElement.replace("\"", "");
			expressionGrammar.setLeftClause(parseExpression(variables, new ExpressionImpl(leftElement, (Variable[]) expr.getVariables().toArray())));
			expressionGrammar.setRightClause(parseExpression(variables, new ExpressionImpl(rightElement, (Variable[]) expr.getVariables().toArray())));
			expressionGrammar.setOperator(ExpressionOperator.or);
		}
		else if(isCorrectOperator(expr.getExpression().split("\\!"))){
			expressionGrammar = new ExpressionComposed();
			String rightElement = expr.getExpression().split("\\!")[1];
			rightElement.replace("\"", "");
			expressionGrammar.setLeftClause(null);
			expressionGrammar.setRightClause(parseExpression(variables, new ExpressionImpl(rightElement, (Variable[]) expr.getVariables().toArray())));
			expressionGrammar.setOperator(ExpressionOperator.negation);
		}
		return expressionGrammar;
		
	}
	
	/**
	 * Not yet support for multiple of the same operators
	 * @param parts
	 * @return
	 */
	private static boolean isCorrectOperator(String[] parts){
		if(parts.length > 2){
			throw new NullPointerException();
		}
		else if(parts.length < 2){
			return false;
		}
		if((parts[0].split("\\(").length == (parts[0].split("\\)").length + 1)) && (parts[1].split("\\(").length == (parts[1].split("\\)").length))){
			return true;
		}
		else{
			return false;
		}
	}
	
	/**
	 * Assumption: independence between the clauses, i.e., (v1 == 1) && (v1 == 1) has a lower probability than (v1 == 1)
	 * @param grammar
	 * @return
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 */
	private static Double determineProbability(ExpressionGrammar grammar) throws IllegalAccessException, InstantiationException{
		if(grammar.getOperator().equals(ExpressionOperator.equals)){
			return determineProbabilityVariableValue(grammar.getVariables().get(0), grammar.getValues().get(0));
		}
		else if(grammar.getOperator().equals(ExpressionOperator.notEquals)){
			return 1 - determineProbabilityVariableValue(grammar.getVariables().get(0), grammar.getValues().get(0));
		}
		else if(grammar.getOperator().equals(ExpressionOperator.and)){
			return determineProbability(grammar.leftClause) * determineProbability(grammar.rightClause);
		}
		else if(grammar.getOperator().equals(ExpressionOperator.or)){
			return determineProbability(grammar.leftClause) + determineProbability(grammar.rightClause);
		}
		else if(grammar.getOperator().equals(ExpressionOperator.negation)){
			return 1 - determineProbability(grammar.rightClause);
		}
		return -1d;
	}
	
	@SuppressWarnings("unchecked")
	private static Double determineProbabilityVariableValue(Variable var, String value) throws IllegalAccessException, InstantiationException{
		String initialValue = (String) var.getIndependentProperty(InitialDataValue.class);
		HashMap<Pair<String, String>, Double> matrix = (HashMap<Pair<String, String>, Double>) var.getIndependentProperty(DataValueMatrix.class);
		// now assume we can reach this in one jump
		Double jumpProb = matrix.get(new Pair<String, String>(initialValue, value));
		if(jumpProb == null){
			return -1.0;
		}
		else{
			return jumpProb;
		}
	}
}
