package org.processmining.plugins.bpsim.elementparameter;

import java.util.Collection;

import org.processmining.plugins.bpsim.extended.DataValueMatrixValue;
import org.processmining.plugins.bpsim.extended.YerkesDodsonInterval;
import org.processmining.plugins.bpsim.parameter.Parameter;
import org.processmining.plugins.bpsim.utility.Util;
import org.processmining.plugins.properties.enumerations.OROptions;

public class ExtendedParameters {
	private Collection<DataValueMatrixValue> dataValueMatrix;
	private Parameter dataValues;
	private Parameter fitness;	
	private OROptions orSemantic;	
	private Parameter throughputTime;
	private Parameter throughputTimeOrder;
	private Parameter utilisation;
	private Parameter waitTime;
	private Parameter waitTimeOrder;	
	private Collection<YerkesDodsonInterval> yerkesDodsonIntervals; 
		
	public ExtendedParameters () {
		
	}
	
	public Collection<DataValueMatrixValue> getDataValueMatrix() {
		return dataValueMatrix;
	}
	public void setDataValueMatrix(Collection<DataValueMatrixValue> dataValueMatrix) {
		this.dataValueMatrix = dataValueMatrix;
	}
	public Parameter getDataValues() {
		return dataValues;
	}
	public void setDataValues(Parameter dataValues) {
		this.dataValues = dataValues;
	}
	public Parameter getFitness() {
		return fitness;
	}
	public void setFitness(Parameter fitness) {
		this.fitness = fitness;
	}
	public OROptions getOrSemantic() {
		return orSemantic;
	}
	public void setOrSemantic(OROptions orSemantic) {
		this.orSemantic = orSemantic;
	}	
	public Parameter getThroughputTime() {
		return throughputTime;
	}
	public void setThroughputTime(Parameter throughputTime) {
		this.throughputTime = throughputTime;
	}
	public Parameter getThroughputTimeOrder() {
		return throughputTimeOrder;
	}
	public void setThroughputTimeOrder(Parameter throughputTimeOrder) {
		this.throughputTimeOrder = throughputTimeOrder;
	}
	public Parameter getUtilisation() {
		return utilisation;
	}
	public void setUtilisation(Parameter utilisation) {
		this.utilisation = utilisation;
	}
	public Parameter getWaitTime() {
		return waitTime;
	}
	public void setWaitTime(Parameter waitTime) {
		this.waitTime = waitTime;
	}
	public Parameter getWaitTimeOrder() {
		return waitTimeOrder;
	}
	public void setWaitTimeOrder(Parameter waitTimeOrder) {
		this.waitTimeOrder = waitTimeOrder;
	}
	public Collection<YerkesDodsonInterval> getYerkesDodsonInterval() {
		return yerkesDodsonIntervals;
	}
	public void setYerkesDodsonInterval(Collection<YerkesDodsonInterval> yerkesDodsonInterval) {
		this.yerkesDodsonIntervals = yerkesDodsonInterval;
	}
	
	public String exportExtendedParameter(int n) {
		String eol = System.getProperty("line.separator");
		String indent = Util.Indent(n);
		
		String export = indent + "<bpsim:ExtendedParameters>" + eol;
								
		n++;
		String indent2 = Util.Indent(n);
				
		if (dataValueMatrix != null){
			export += indent2 + "<bpsim:DataValueMatrix>" + eol;
			for (DataValueMatrixValue dvm: dataValueMatrix) {
				export += dvm.exportDataValueMatrixValue(n+1);
			}
			export += indent2 + "</bpsim:DataValueMatrix>" + eol;
		}
		
		if (dataValues != null){						
			export += indent2 + "<bpsim:DataValues>" + eol;
			//parameter value
			export += dataValues.exportParameter(n+1);
			
			export += indent2 + "</bpsim:DataValues>" + eol;
		}
		
		if (fitness != null){						
			export += indent2 + "<bpsim:Fitness>" + eol;
			//parameter value
			export += fitness.exportParameter(n+1);
			
			export += indent2 + "</bpsim:Fitness>" + eol;
		}								
		
		if (orSemantic != null){	
			export += indent2 + "<bpsim:OrSemantic>" + orSemantic + "</bpsim:OrSemantic>" + eol;		
		}
		
		if (throughputTime != null){						
			export += indent2 + "<bpsim:ThroughputTime>" + eol;
			//parameter value
			export += throughputTime.exportParameter(n+1);
			
			export += indent2 + "</bpsim:ThroughputTime>" + eol;
		}
		
		if (throughputTimeOrder != null){						
			export += indent2 + "<bpsim:ThroughputTimeOrder>" + eol;
			//parameter value
			export += throughputTimeOrder.exportParameter(n+1);
			
			export += indent2 + "</bpsim:ThroughputTimeOrder>" + eol;
		}
		
		if (utilisation != null){						
			export += indent2 + "<bpsim:Utilisation>" + eol;
			//parameter value
			export += utilisation.exportParameter(n+1);
			
			export += indent2 + "</bpsim:Utilisation>" + eol;
		}
		
		if (waitTime != null){						
			export += indent2 + "<bpsim:WaitTime>" + eol;
			//parameter value
			export += waitTime.exportParameter(n+1);
			
			export += indent2 + "</bpsim:WaitTime>" + eol;
		}
		
		if (waitTimeOrder != null){						
			export += indent2 + "<bpsim:WaitTimeOrder>" + eol;
			//parameter value
			export += waitTimeOrder.exportParameter(n+1);
			
			export += indent2 + "</bpsim:WaitTimeOrder>" + eol;
		}
		
		
		if (yerkesDodsonIntervals != null){
			export += indent2 + "<bpsim:YerkesDodsonIntervals>" + eol;
			for (YerkesDodsonInterval ydi: yerkesDodsonIntervals) {
				export += ydi.exportYerkesDodsonInterval(n+1);
			}
			export += indent2 + "</bpsim:YerkesDodsonIntervals>" + eol;
		}
		
		export += indent + "</bpsim:ExtendedParameters>" + eol;					
		return export;
	}


	
	
	
	
	

}
