package org.processmining.plugins.bpsim.scenario;

import java.util.Collection;
import java.util.HashSet;
import java.util.UUID;

import org.processmining.plugins.bpsim.elementparameter.ControlParameters;
import org.processmining.plugins.bpsim.elementparameter.CostParameters;
import org.processmining.plugins.bpsim.elementparameter.ExtendedParameters;
import org.processmining.plugins.bpsim.elementparameter.PriorityParameters;
import org.processmining.plugins.bpsim.elementparameter.ResourceParameters;
import org.processmining.plugins.bpsim.elementparameter.TimeParameters;
import org.processmining.plugins.bpsim.utility.Util;


public class ElementParameters {
	private String id;
		
	private UUID elementRefID;
	private String elementRefNodeID;

	private PropertyParameters propertyParameters;
	private Collection<VendorExtension> vendorExtension;
	
	private TimeParameters timeParameters;
	private ControlParameters controlParameters;
	private ResourceParameters resourceParameters;
	private CostParameters costParameters;
	private PriorityParameters priorityParameters;
	private ExtendedParameters extendedParameters;		

	public ElementParameters() {
		id = UUID.randomUUID().toString();
				
		vendorExtension = new HashSet<VendorExtension>();				
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}
	
	public UUID getElementRefID() {
		return elementRefID;
	}

	public void setElementRefID(UUID elementRefID) {
		this.elementRefID = elementRefID;
	}

	public PropertyParameters getPropertyParameters() {
		return propertyParameters;
	}

	public void setPropertyParameters(PropertyParameters propertyParameters) {
		this.propertyParameters = propertyParameters;
	}

	public Collection<VendorExtension> getVendorExtension() {
		return vendorExtension;
	}

	public void setVendorExtension(Collection<VendorExtension> vendorExtension) {
		this.vendorExtension = vendorExtension;
	}

	public TimeParameters getTimeParameters() {
		return timeParameters;
	}

	public void setTimeParameters(TimeParameters timeParameters) {
		this.timeParameters = timeParameters;
	}

	public ControlParameters getControlParameters() {
		return controlParameters;
	}

	public void setControlParameters(ControlParameters controlParameters) {
		this.controlParameters = controlParameters;
	}

	public ResourceParameters getResourceParameters() {
		return resourceParameters;
	}

	public void setResourceParameters(ResourceParameters resourceParameters) {
		this.resourceParameters = resourceParameters;
	}

	public CostParameters getCostParameters() {
		return costParameters;
	}

	public void setCostParameters(CostParameters costParameters) {
		this.costParameters = costParameters;
	}

	public PriorityParameters getPriorityParameters() {
		return priorityParameters;
	}

	public void setPriorityParameters(PriorityParameters priorityParameters) {
		this.priorityParameters = priorityParameters;
	}
	
	public ExtendedParameters getExtendedParameters() {
		return extendedParameters;
	}

	public void setExtendedParameters(ExtendedParameters extendedParameters) {
		this.extendedParameters = extendedParameters;
	}
	
	public String exportElementParameter(int n, boolean[] parameters) {
		String export = "";
		if (elementRefNodeID != null || parameters[1]) {
			String eol = System.getProperty("line.separator");
			String indent = Util.Indent(n);
									
			//parameters
			if (timeParameters != null){
				export += timeParameters.exportTimeParameter(n+1);
			}
			
			if (controlParameters != null){			
				export += controlParameters.exportControlParameter(n+1);
			}
			
			if (costParameters != null){			
				export += costParameters.exportCostParameter(n+1);
			}
			
			if (resourceParameters != null){			
				export += resourceParameters.exportResourceParameter(n+1);
			}
			
			if (priorityParameters != null){			
				export += priorityParameters.exportPriorityParameter(n+1);
			}
			
			if (propertyParameters != null){			
				export += propertyParameters.exportPropertyParameters(n+1);
			}
					
			if (extendedParameters != null && parameters[0]){			
				export += extendedParameters.exportExtendedParameter(n+1);
			}
					
			for (VendorExtension vendorextension: vendorExtension) {
				export += vendorextension.exportVendorExtension(n+1);
			}
						
			if (!export.isEmpty()) {				
				String export2 = "";
				export2 = indent + "<bpsim:ElementParameters";						
				
				if (elementRefID != null){
					if (elementRefNodeID != null){
						export2 += " elementRef=\"" + elementRefNodeID.toString().replaceAll(" ", "_").toLowerCase() + "\"";
					} else {
						export2 += " elementRef=\"" + elementRefID.toString() + "\"";
					}
										
					export2 += ">" + eol;
				}
				else{
					if (elementRefNodeID != null){
						export2 += " elementRef=\"" + elementRefNodeID.toString().replaceAll(" ", "_").toLowerCase() + "\"";
					}
					export2 += ">" + eol;
				}
				
				export = export2 + export;
				export += indent + "</bpsim:ElementParameters>" + eol;		
			}
														
		}
		return export;
	}
	
	public String getElementRefnodeID() {
		return elementRefNodeID;
	}

	public void setElementRefnodeID(String elementRefNodeID) {
		this.elementRefNodeID = elementRefNodeID;
	}

}
