package org.processmining.plugins.bpsim.scenario;

import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.UUID;

import org.processmining.plugins.bpsim.parameter.BPSimCalendar;
import org.processmining.plugins.bpsim.utility.Util;

public class Scenario {
	private String id;	
	private String name;
	private String description;
	private Date created;
	private Date modified;
	private String author;
	private String vendor;
	private String version;
	
	private Scenario result;
	private Scenario inherits;
	
	private ScenarioParameters scenarioParameter;
	private Collection<ElementParameters> elementParameter;
	private Collection<VendorExtension> vendorExtension;
	private Collection<BPSimCalendar> calendar;
	
	final private String ISO_FORMAT = "yyyy-MM-dd'T'HH:mm:ss";
	final private SimpleDateFormat sdf = new SimpleDateFormat(ISO_FORMAT);
	
	public Scenario() {
		id = UUID.randomUUID().toString();
			
		Date now = new Date();		
		created = now;
		modified = now;
		
		elementParameter = new HashSet<ElementParameters>();
		vendorExtension = new HashSet<VendorExtension>();
		calendar = new HashSet<BPSimCalendar>();
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Date getCreated() {
		return created;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public Date getModified() {
		return modified;
	}

	public void setModified(Date modified) {
		this.modified = modified;
	}

	public String getAuthor() {
		return author;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public String getVendor() {
		return vendor;
	}

	public void setVendor(String vendor) {
		this.vendor = vendor;
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public Scenario getResult() {
		return result;
	}

	public void setResult(Scenario result) {
		this.result = result;
	}

	public Scenario getInherits() {
		return inherits;
	}

	public void setInherits(Scenario inherits) {
		this.inherits = inherits;
	}

	public ScenarioParameters getScenarioParameter() {
		return scenarioParameter;
	}

	public void setScenarioParameter(ScenarioParameters scenarioParameter) {
		this.scenarioParameter = scenarioParameter;
	}

	public Collection<ElementParameters> getElementParameter() {
		return elementParameter;
	}

	public void setElementParameter(Collection<ElementParameters> elementParameter) {
		this.elementParameter = elementParameter;
	}

	public Collection<VendorExtension> getVendorExtension() {
		return vendorExtension;
	}

	public void setVendorExtension(Collection<VendorExtension> vendorExtension) {
		this.vendorExtension = vendorExtension;
	}

	public Collection<BPSimCalendar> getCalendar() {
		return calendar;
	}

	public void setCalendar(Collection<BPSimCalendar> calendar) {
		this.calendar = calendar;
	}

	public String exportScenario(int n, boolean[] parameters) {
		String eol = System.getProperty("line.separator");
		String indent = Util.Indent(n);
		
		String export = indent + "<bpsim:Scenario";

		if (author != null){
			export += " author=\"" + author + "\"";
		}
		
		if (created != null){
			export += " created=\"" + sdf.format(created) + "\"";
		}
		
		if (id != null){
			id = " " + id;
			export += " id=\"" + id.toString().replaceAll(" ", "_").toLowerCase() + "\"";
		}
		
		if (modified != null){
			
			export += " modified=\"" + sdf.format(modified) + "\"";
		}
		
		if (name != null){
			export += " name=\"" + name + "\"";
		}
		
		if (description != null){
			export += " description=\"" + description + "\"";
		}
		
		if (vendor != null){
			export += " vendor=\"" + vendor + "\"";
		}
		
		if (version != null){
			export += " version=\"" + version + "\"";
		}
		
		export += ">" + eol;
		
		//result
		if (result != null) {
			export += result.exportScenario(n+1, parameters);
		}
		
		//inherits
		if (inherits != null) {
			export += inherits.exportScenario(n+1, parameters);
		}
				
		//scenario parameters
		if (scenarioParameter != null) {
			export += scenarioParameter.exportScenarioParameters(n+1, parameters);
		}
		
		//element parameters
		for (ElementParameters ep: elementParameter) {
			export += ep.exportElementParameter(n+1, parameters);
		}
		
		//vendor
		for (VendorExtension vendorextension: vendorExtension) {
			export += vendorextension.exportVendorExtension(n+1);
		}
				
		//calendars
		for (BPSimCalendar cal: calendar) {
			export += cal.exportBPSimCalendar(n+1);
		}
		
		export += indent + "</bpsim:Scenario>" + eol;					
		return export;
	}

	
}
