package org.processmining.plugins.bpsim.scenario;

import java.util.Collection;
import java.util.HashSet;

import org.processmining.plugins.bpsim.parameter.Parameter;
import org.processmining.plugins.bpsim.utility.Util;
import org.processmining.plugins.properties.enumerations.QueueType;


public class ScenarioParameters {
	private Parameter start;
	private Parameter duration;
	private int replication;
	private long seed;
	private TimeUnit baseTimeUnit;
	private String baseCurrencyUnit;	
	
	//extended bpsim parameters
	private boolean push;
	private boolean pull;
	private QueueType qType;
	private Parameter warmUpPeriod;
	
	private Collection<PropertyParameters> propertyParameters;
	
	public ScenarioParameters() {
		propertyParameters = new HashSet<PropertyParameters>();
	}

	public Parameter getStart() {
		return start;
	}

	public void setStart(Parameter start) {
		this.start = start;
	}

	public Parameter getDuration() {
		return duration;
	}

	public void setDuration(Parameter duration) {
		this.duration = duration;
	}

	public int getReplication() {
		return replication;
	}

	public void setReplication(int replication) {
		this.replication = replication;
	}

	public long getSeed() {
		return seed;
	}

	public void setSeed(long seed) {
		this.seed = seed;
	}

	public TimeUnit getBaseTimeUnit() {
		return baseTimeUnit;
	}

	public void setBaseTimeUnit(TimeUnit baseTimeUnit) {
		this.baseTimeUnit = baseTimeUnit;
	}

	public String getBaseCurrencyUnit() {
		return baseCurrencyUnit;
	}

	public void setBaseCurrencyUnit(String baseCurrencyUnit) {
		this.baseCurrencyUnit = baseCurrencyUnit;
	}

	public Collection<PropertyParameters> getPropertyParameters() {
		return propertyParameters;
	}

	public void setPropertyParameters(Collection<PropertyParameters> propertyParameters) {
		this.propertyParameters = propertyParameters;
	}
	
	public boolean isPush() {
		return push;
	}

	public void setPush(boolean push) {
		if (push) {
			this.push = push;
		} else {
			this.pull = !push;
		}
	}

	public QueueType getqType() {
		return qType;
	}

	public void setqType(QueueType qType) {
		this.qType = qType;
	}

	public Parameter getWarmUpPeriod() {
		return warmUpPeriod;
	}

	public void setWarmUpPeriod(Parameter warmUpPeriod) {
		this.warmUpPeriod = warmUpPeriod;
	}
	
	public String exportScenarioParameters(int n, boolean[] parameters) {
		String eol = System.getProperty("line.separator");
		String indent = Util.Indent(n);
		
		String export = indent + "<bpsim:ScenarioParameters";		

		if (replication != 0){
			export += " replication=\"" + replication + "\"";
		}
		
		if (seed != 0){
		}
		
		if (baseTimeUnit != null){		
			export += " baseTimeUnit=\"" + baseTimeUnit + "\"";
		}
		
		if (baseCurrencyUnit != null){
			export += " baseCurrencyUnit=\"" + baseCurrencyUnit + "\"";
		}
		
		//extended
		if (qType != null && parameters[0]){
			export += " qType=\"" + qType + "\"";
		}
		
		//extended
		if (push && parameters[0]){
			export += " Push=\"" + push + "\"";
		}
		
		//extended
		if (pull && parameters[0]){
			export += " Pull=\"" + pull + "\"";
		}
		
		export += ">" + eol;	
		n++;		
		String indent2 = Util.Indent(n);
		
		if (start != null){
			export += indent2 + "<bpsim:Start>" + eol;			
			export += start.exportParameter(n+1);
			export += indent2 + "</bpsim:Start>" + eol;
		}
		
		if (duration != null){
			export += indent2 + "<bpsim:Duration>" + eol;			
			export += duration.exportParameter(n+1);
			export += indent2 + "</bpsim:Duration>" + eol;
		}
			
		//extended
		if (warmUpPeriod != null && parameters[0]){
			export += indent2 + "<bpsim:WarmUpPeriod>" + eol;			
			export += warmUpPeriod.exportParameter(n+1);
			export += indent2 + "</bpsim:WarmUpPeriod>" + eol;
		}
		
		for (PropertyParameters pp : propertyParameters) {
			export += pp.exportPropertyParameters(n+1);
		}
													
		export += indent + "</bpsim:ScenarioParameters>" + eol;					
		return export;
	}
	
}
